import json
from django.core.serializers.json import DjangoJSONEncoder
from django.db import models
from django.http import HttpResponseRedirect
from wagtail.admin.panels import (
    TabbedInterface, ObjectList, FieldPanel, InlinePanel, MultiFieldPanel, FieldRowPanel, HelpPanel,
    PublishingPanel,MultipleChooserPanel
)

from wagtail.fields import StreamField, RichTextField
from wagtail.models import Page,  PreviewableMixin, Orderable, PageManager
from wagtail.search import index
from wagtail.snippets.models import register_snippet, register_deferred_snippets
from modelcluster.models import ClusterableModel
from modelcluster.fields import ParentalKey
from modelcluster.contrib.taggit import ClusterTaggableManager
from taggit.models import TagBase, GenericTaggedItemBase, ItemBase


from wagtail_trash.managers import TrashManager
from wagtail.blocks import *
from wagtailmodelchooser.blocks import ModelChooserBlock

from django.contrib.auth.models import User

from wagtailmodelchooser import register_model_chooser

from django.db.models import Q

from datetime import date, datetime
from pytz import timezone
from django import forms

from django_extensions.db.fields import AutoSlugField
from django.template.defaultfilters import slugify

from django_ckeditor_5.fields import CKEditor5Field
from django_countries.fields import CountryField
from modelcluster.contrib.taggit import ClusterTaggableManager
from taggit.models import TagBase, ItemBase
from wagtail_color_panel.fields import ColorField
from wagtail_color_panel.edit_handlers import NativeColorPanel
# from wagtail.images.blocks import ImageChooserBlock
# from wagtail.images.blocks import ImageBlock

from wagtailmetadata.models import MetadataPageMixin
from wagtail.contrib.routable_page.models import RoutablePageMixin, path, re_path

from core.common import  MONTHS
from core.abstract import AbstractCarousel, AbstractCustomColor
from core.blocks import WebStoryBlock
from .panels import SubcategoryChooser
from core.models import SocialPlatform,CustomTag

from wagtail.search import index
from django.http import JsonResponse
from wagtail.admin.forms import WagtailAdminPageForm
from hitcount.models import HitCountMixin, HitCount
from modelcluster.fields import ParentalManyToManyField


class Industry(models.Model):
    name=models.CharField(max_length=200)
    desc=RichTextField(blank=True,null=True)
    def __str__(self):
        return self.name
    class Meta:
        verbose_name_plural="Industries"

class BaseAttributes(ClusterableModel):
    name = models.CharField(null=True, max_length=255,verbose_name="Attribute")
    icon = models.CharField(max_length=250, verbose_name="Icon", blank=True, null=True)
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel("name",heading="Group Name"),
                FieldPanel("icon",heading="Group Icon"),
            ]),
        ]),
        InlinePanel("attr_items"),
    ]
    def __str__(self):
        return str(self.name)
    class Meta:
        verbose_name_plural = "Common Attributes"
        verbose_name = "Common Attribute"

class BaseAttrItems(Orderable):
    group = ParentalKey(BaseAttributes, related_name="attr_items")
    name = models.CharField(null=True, max_length=255,
                            verbose_name="Attribute")
    slug = AutoSlugField(populate_from='name', editable=True)
    
    unit = models.CharField(
        max_length=250, verbose_name="Unit", blank=True, null=True)
    icon = models.CharField(
        max_length=250, verbose_name="Icon", blank=True, null=True)
    
    def slugify_function(self, content):
        slug = content.replace(' ', '-').lower()
        return slug.replace('_', '-').lower()

    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel("name"),
                FieldPanel("slug"),
                FieldPanel("unit"),
                FieldPanel("icon"),
            ])
        ])
    ]

    def __str__(self):
        if self.unit != None:
            return str(self.group)+ " > " +str(self.name)+" ("+str(self.unit)+")"
        else:
            return str(self.group)+ " > " +str(self.name)

    
class Category(ClusterableModel, AbstractCustomColor):
    name = models.CharField(null=True, max_length=255, verbose_name="Name")
    slug = AutoSlugField(populate_from='name', editable=True)
    icon = models.CharField(null=True, blank=True,
                            max_length=255, verbose_name="Icon")
    catImage = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.SET_NULL, related_name='+', blank=True, null=True, verbose_name="Category Image"
    )
    order = models.IntegerField(
        unique=True, db_index=True, null=True, blank=True)
    show_in_mobile_menu = models.BooleanField(
        default=False, verbose_name="Show in mobile menu")

    def slugify_function(self, content):
        slug = content.replace(' ', '-').lower()
        return slug.replace('_', '-').lower()

    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('name'),
                FieldPanel("slug"),
            ]),
        ]),
        
        # MultiFieldPanel([
        #     FieldRowPanel([
        #         NativeColorPanel('bgColor'),
        #         NativeColorPanel('fgColor'),
        #     ]),
        #     FieldRowPanel([
        #         NativeColorPanel('hoverBgColor'),
        #         NativeColorPanel('hoverFgColor'),
        #     ]),
        #     FieldRowPanel([
        #         NativeColorPanel('activeBgColor'),
        #         NativeColorPanel('activeFgColor'),
        #     ]),
        # ], heading="Sub Category Tag Colors"),
        
        
        InlinePanel('sub_categories', label="Sub Category"),
        InlinePanel('price_range', label="Price Range"),
        InlinePanel('features', label="Features")
        # InlinePanel('cat_types', label="Category Type")
    ]

    def __str__(self):
        return f'{self.name}'

    class Meta:
        ordering = ["name"]
        verbose_name_plural = "Category (Menu)"


class SubCategory(Orderable, AbstractCustomColor):
    cat = ParentalKey(Category, related_name="sub_categories")
    name = models.CharField(null=True, max_length=255, verbose_name="Name")
    slug = AutoSlugField(populate_from='name',editable=True)
    icon = models.CharField(null=True, blank=True,
                            max_length=255, verbose_name="Icon")
    newProduct = models.IntegerField(null=True, blank=True,verbose_name="New Product (Month)",default=1,choices=MONTHS)
    image = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.SET_NULL, related_name='+', blank=True, null=True, verbose_name="Image"
    )
    newBgColor = ColorField(
        default="#FFFFFF", verbose_name="New BG", blank=True, null=True)
    newFgColor = ColorField(
        default="#000000", verbose_name="New FG", blank=True, null=True)
    upBgColor = ColorField(
        default="#FFFFFF", verbose_name="Upcoming BG", blank=True, null=True)
    upFgColor = ColorField(
        default="#000000", verbose_name="Upcoming FG", blank=True, null=True)

    def slugify_function(self, content):
        slug = content.replace(' ', '-').lower()
        return slug.replace('_', '-').lower()
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('name'),
                FieldPanel('slug')
            ]),
            FieldRowPanel([
                FieldPanel('newProduct'),
                FieldPanel('image')
            ]),
            FieldRowPanel([
                NativeColorPanel('newBgColor'),
                NativeColorPanel('newFgColor'),
                NativeColorPanel('upBgColor'),
                NativeColorPanel('upFgColor'),
            ])
        ]),
    ]

    def __str__(self):
        return f'{self.name}'

    class Meta:
        verbose_name_plural = "Category"

class PriceRange(Orderable):
    cat = ParentalKey(Category, related_name="price_range")
    title = models.CharField('Title',
                             max_length=200, null=True, blank=True)
    minPrice=models.PositiveIntegerField()
    maxPrice=models.PositiveIntegerField(null=True,blank=True)
    panels = [
        MultiFieldPanel([
                FieldPanel('title'),
            FieldRowPanel([
                FieldPanel('minPrice'),
                FieldPanel('maxPrice')
            ])
        ])
    ]
    def __str__(self):
        return str(self.title)

class CatFeature(Orderable):
    cat = ParentalKey(Category, related_name="features")
    feature = models.ForeignKey(BaseAttrItems, on_delete=models.CASCADE, null=True,verbose_name="")
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                 SubcategoryChooser(
                    "feature",
                    category=BaseAttributes,
                    subcategory=BaseAttrItems,
                    subcategory_related_name="attr_items",
                    search_text="Search Attributes"
                ),
            ])
        ])
    ]


class ProdFilter(ClusterableModel): # Note Used
    cat = models.ForeignKey(
        Category, on_delete=models.CASCADE, null=True, related_name="cat_filters")
    name = models.CharField(
        max_length=255, verbose_name="Name")
    show_in_menu = models.BooleanField(default=True)
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel("cat"),
                FieldPanel("name"),
            ]),
            FieldPanel("show_in_menu"),
        ]),
        InlinePanel('filter_items', label="Filter Items"),
    ]

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = 'Product Filters'

class FilterItems(Orderable):# Note Used
    filter = ParentalKey(ProdFilter, related_name="filter_items")
    title = models.CharField('Title',
                             max_length=200, null=True, blank=True)

    value = models.CharField(
        'Filter Value', max_length=200, null=True, blank=True)
    show_in_menu = models.BooleanField(default=True)

    panels = [
        FieldPanel('title'),
        FieldPanel('value'),
        FieldPanel("show_in_menu"),
    ]

    def __str__(self):
        return str(self.title)


def slugify_function(self, content):
    slug = content.replace(' ', '_').lower()
    return slug

class Company(ClusterableModel):
    industry = models.ForeignKey(
        Industry, on_delete=models.SET_NULL, null=True, related_name="companies")
    coName = models.CharField(max_length=200, verbose_name="Name")
    alias = models.CharField(max_length=200, verbose_name="Alias Name",null=True,blank=True)
    coOrigin = CountryField(blank=True, null=True, verbose_name="Origin")
    coWeb = models.URLField(blank=True, null=True, verbose_name="Website")
    coDesc = RichTextField(blank=True, null=True, verbose_name="Description")
    coFounded = models.DateField(
        blank=True, null=True, verbose_name="Founded Date")
    parent = models.ForeignKey(
        'self', on_delete=models.CASCADE, null=True, blank=True, related_name='parents')
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('industry'),
                FieldPanel('coOrigin'),
            ]),
            FieldPanel('coName'),
            FieldRowPanel([
                FieldPanel('alias'),
                FieldPanel('parent'),
            ]),
        ]),
        
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('coFounded'),
                FieldPanel('coWeb'),
            ]),
            FieldPanel('coDesc'),
        ],heading="Details"),
        # MultiFieldPanel([
        # InlinePanel('brands')
        # ],heading="Brands")
    ]
    def brand(self):
        brands=""
        for br in self.brands.all():
            brands+=br.name+", "
        brands+="( "+self.coName+" )"
        return brands
    
    def __str__(self):
        return self.coName

class Brands(Orderable):
    com = ParentalKey(Company, related_name="brands")
    name = models.CharField(null=True, max_length=255, verbose_name="Name")
    slug = AutoSlugField(populate_from='name', editable=True)
    image = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.SET_NULL, related_name='+', blank=True, null=True, verbose_name="Image"
    )
    categories=models.ManyToManyField(Category,blank=True,related_name="brand")
    brDesc = RichTextField(blank=True, null=True, verbose_name="Description")
    brWeb = models.URLField(blank=True, null=True, verbose_name="Website")
    def slugify_function(self, content):
        slug = content.replace(' ', '-').lower()
        return slug.replace('_', '-').lower()
    panels = [
        FieldPanel('com',heading="Company"),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('name'),
                FieldPanel('slug'),
                FieldPanel('image'),
            ]),
        ]),
        FieldPanel("categories", widget=forms.CheckboxSelectMultiple)
    ]
    def __str__(self):
        return f'{self.name}'
    


class ProductType(ClusterableModel): # Product Attributes Category Wise
    cat = models.ForeignKey(SubCategory, on_delete=models.SET_NULL, null=True,verbose_name="Category")
    type = models.CharField(null=True, max_length=255, verbose_name="Title")
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                SubcategoryChooser(
                    "cat",
                    category=Category,
                    subcategory=SubCategory,
                    subcategory_related_name="sub_categories",
                    search_text="Search Categories"
                ),
                FieldPanel("type"),
            ])
        ]),
        InlinePanel('type_attributes', label="Attribute")
    ]
    def __str__(self):
        return f'{self.type}'
    class Meta:
        verbose_name_plural = "Product Type"

class ProductAttributes(Orderable):
    type = ParentalKey(ProductType, related_name="type_attributes")
    attr = models.ForeignKey(
        BaseAttrItems, on_delete=models.CASCADE, null=True,related_name="attr_items",verbose_name="")
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                 SubcategoryChooser(
                    "attr",
                    category=BaseAttributes,
                    subcategory=BaseAttrItems,
                    subcategory_related_name="attr_items",
                    search_text="Search Attributes"
                ),
            ])
        ])
    ]
    @property
    def prodWidget(self):
        if self.attrType == 'Checkbox':
            return forms.CheckboxInput()
        elif self.attrType == 'List':
            return forms.RadioSelect()

    def __str__(self):
        return str(self.attr)

    class Meta:
        verbose_name_plural = "Product Attributes"


class App(ClusterableModel):
    cat=models.ForeignKey(ProductAttributes,on_delete=models.SET_NULL,null=True,blank=True,verbose_name="Category")
    brand = models.ForeignKey(Brands, on_delete=models.SET_NULL, null=True,blank=True, related_name="br_apps", verbose_name="Brand")
    series=models.CharField(max_length=200,blank=True,null=True)
    family=models.CharField(max_length=200,verbose_name="Family",blank=True,null=True)
    fromDate=models.DateField(blank=True,null=True,verbose_name="From Date")
    toDate=models.DateField(blank=True,null=True,verbose_name="To Date")
    image = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.SET_NULL, related_name='+', blank=True, null=True, verbose_name="Image"
    )
    panels=[
    MultiFieldPanel([
        FieldRowPanel([
        SubcategoryChooser(
        "cat",
        category=ProductType,
        subcategory=ProductAttributes,
        subcategory_related_name="type_attributes",
        search_text="Search Categories"
        ),
        SubcategoryChooser(
            "brand",
            category=Company,
            subcategory=Brands,
            subcategory_related_name="brands",
            search_text="Search Brands"
        ),
        ]),
        FieldRowPanel([
        FieldPanel('series'),
        FieldPanel('family'),
        ]),
        FieldPanel('image'),
        InlinePanel("app_items")
    ]),
    ]
    def __str__(self):
        return str(self.brand)
    class Meta:
        verbose_name_plural="App"

class AppItems(Orderable):
    app=ParentalKey(App,related_name="app_items")
    osName=models.CharField(max_length=200,verbose_name="Name",blank=True,null=True)
    version=models.CharField(max_length=200)
    osfamily=models.CharField(max_length=200,verbose_name="Name",blank=True,null=True)
    osfamilyVersion=models.CharField(max_length=200,verbose_name="Version")
    releaseDate=models.DateField(blank=True,null=True,verbose_name="Release Date")
    features=RichTextField(blank=True,null=True)
    image = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.SET_NULL, related_name='+', blank=True, null=True, verbose_name="Image"
    )
    panels=[
    MultiFieldPanel([
        FieldRowPanel([
            FieldPanel('version',heading="App Version"),
            FieldPanel('osfamilyVersion',heading="Family Version"),
            FieldPanel('releaseDate'),
        ]),
        FieldPanel('features'),
    FieldRowPanel([
            FieldPanel('osName',heading="App Name"),
            FieldPanel('osfamily',heading="Family Name"),
        ]),
    ]),
    ]
    def __str__(self):
        return str(self.osName)+" "+str(self.version)



class Recommendation(models.Model):
    recText = models.CharField(max_length=200, verbose_name="Text")
    recImg = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.SET_NULL, related_name='+', blank=True, null=True, verbose_name="Image"
    )
    recBgColor = ColorField(default="#000000", verbose_name="BG")
    recFgColor = ColorField(default="#FFFFFF", verbose_name="FG")
    panels = [
        FieldPanel('recText'),
        FieldPanel('recImg'),
        MultiFieldPanel([
            FieldRowPanel([
                NativeColorPanel('recBgColor'),
                NativeColorPanel('recFgColor'),
            ])
        ], heading="Color")
    ]

    def __str__(self):
        return self.recText


class CategoryIndexPage(RoutablePageMixin, MetadataPageMixin, AbstractCarousel, Page):
    template="ListPage/ecom/CatIndex.html"
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    page_description = " Categories Index Page Grid & List View"
    menuImageText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Icon Text")
    #Not Used
    intro = RichTextField(blank=True)
    show_footer = models.BooleanField(blank=True, null=True, default=True)
    subpage_types = ['ecom.CategoryPage']
    content_panels = Page.content_panels + [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('search_image',heading="Thumb Image"),
                FieldPanel('menuImageText')
            ])
        ]),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('bannerHeight'),
                FieldPanel('bannerWidth'),
                FieldPanel('bannerTimer'),
            ]),
            FieldPanel('bannercontent')
        ], heading="Banner"),
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(Page.promote_panels, heading='Promote'),
    ])
    def get_child_pages(self):
        qs = CategoryPage.objects.all()
        return qs

    def get_child_categories(self):
        qs = CategoryPage.objects.child_of(self)
        # ThingDetailPage.objects.child_of(self)
        qs = qs.order_by('-last_published_at')
        return qs

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def mobile_breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs[3:]

    def get_absolute_url(self):
        return f'{self.url}'
    @re_path(r'^<cat>/$')
    def all_prod(self, request, cat):
        events = ProductPage.objects.filter(cat__cat__name=cat)
        return self.render(request, "ecom/category_page.html", {
            'events': events,
        })

class CategoryPageTag(ItemBase):
    tag = models.ForeignKey(
        CustomTag, related_name="tagged_cats", on_delete=models.CASCADE
    )
    content_object = ParentalKey(
        'CategoryPage',
        related_name='tagged_items',
        on_delete=models.CASCADE
    )

class CategoryPage(RoutablePageMixin, MetadataPageMixin, AbstractCarousel, Page, AbstractCustomColor, HitCountMixin):
    template="ListPage/ecom/CatList.html"
    page_description = "Add Category e.g: SmartPhone, Table, Wearable, etc..."
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    cat = models.ForeignKey(Category, on_delete=models.PROTECT,
                            null=True, related_name="categories", verbose_name="Category")
    menuName = models.CharField(
        max_length=100, null=True, blank=True, verbose_name="Menu Name")
    menuImageText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Menu Icon Text")
    show_footer = models.BooleanField(blank=True, null=True, default=True)
    # Related News & Blogs
    show_news = models.BooleanField(blank=True, null=True, default=True)
    noOfNews = models.PositiveIntegerField(blank=True, null=True, default=10)
    show_blogs = models.BooleanField(blank=True, null=True, default=True)
    noOfBlogs = models.PositiveIntegerField(blank=True, null=True, default=10)

    # Not Used
    tags = ClusterTaggableManager(through=CategoryPageTag, blank=True)
    menuImage = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.SET_NULL, related_name='+', blank=True, null=True, verbose_name="Menu Image"
    )
    catShortDesc = RichTextField(
        blank=True, null=True, verbose_name="Short Description")
    
    subpage_types = [
        'ecom.ProductBrandPage'
    ]
    content_panels = Page.content_panels + [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('cat'),
                FieldPanel('search_image', heading="Title Image")
            ]),
            FieldRowPanel([
                FieldPanel('menuName'),
                FieldPanel('menuImageText'),
                FieldPanel('show_footer',widget=forms.CheckboxInput())
            ])
        ]),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('bannerHeight'),
                FieldPanel('bannerWidth'),
                FieldPanel('bannerTimer'),
            ]),
            FieldPanel('bannercontent')
        ], heading="Banner"),
    ]
    news_panels=[
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('show_news',widget=forms.CheckboxInput()),
                FieldPanel('noOfNews',heading="No of News"),
            ])
        ],heading="News"),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('show_blogs',widget=forms.CheckboxInput()),
                FieldPanel('noOfBlogs',heading="No of Blogs"),
            ])
        ],heading="Blogs")
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(news_panels, heading='Related News & Blogs'),
        ObjectList(Page.promote_panels, heading='Promote'),
    ])
    
    def get_context(self, request):
        context = super().get_context(request)
        # products = ProductPage.objects.filter(
        #     Q(subCat__cat__slug=self.cat.slug), ~Q(search_image=None))
        prodPages=ProductPage.objects.filter(subCat__cat__slug=self.cat.slug).live()
        
        price_range = request.GET.get('price_range')
        q = request.GET.get('q')
        if q:
            products = ProductPage.objects.filter(subCat__slug=q)
            totalProdList=[]
            for product in products:
                for var in product.get_children().live():
                    if var.specific.thumb:
                        totalProdList.append(var)
            context['total_products'] = totalProdList
        elif price_range:
            min_price, max_price = map(int, price_range.split('-'))
            products = prodPages.filter(
                prodDealPrice__gte=min_price * 1000, prodDealPrice__lt=max_price * 1000)
        else:
            totalProdList=[]
            for product in prodPages:
                for var in product.get_children().live():
                    if var.specific.thumb:
                        totalProdList.append(var)
            print("totalProdList=",totalProdList)
            context['total_products'] = totalProdList
                    
        # context['filters'] = ProdFilter.objects.filter(
        #     show_in_menu=True, cat__slug=self.slug)
       
        return context

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def mobile_breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs[4:]

    def get_absolute_url(self):
        return f'{self.url}'

    # @re_path(r'^<cat>/$')
    # # @path('')
    # # @path('<subcat>/')
    # def all_prod(self, request,subcat=None):
    #     if subcat:
    #         print("Route Query=",subcat)
    #         # try:
    #         events = ProductPage.objects.filter(subCat__slug=subcat)
    #         # except:
    #             # events = CategoryPage.objects.get(title=subcat).get_children()

    #     else:
    #         events = ProductPage.objects.filter(subCat__slug=self.title)

    #     print("Result Set=",events)

    #     return self.render(request, context_overrides={
    #         'events': events,
    #         'title':subcat
    #     },)

    # @path('')
    # def all_prod(self, request):
    #     events = ProductPage.objects.filter(cat__cat__name=self.title)
    #     # return self.render(request,'pages/search_result.html',{'prodObj':events})
    #     return self.render(request, context_overrides={
    #         'events': events,
    #     },)

    class Meta:
        verbose_name = "Category"


class ProductBrandPage(RoutablePageMixin, MetadataPageMixin, AbstractCarousel, Page, AbstractCustomColor, HitCountMixin):
    template="ListPage/ecom/BrandList.html"
    page_description = "Add Category Brands"
    objects = PageManager() 
    objects_excluding_trash = TrashManager()
    show_footer = models.BooleanField(blank=True, null=True, default=True)
    menuName = models.CharField(
        max_length=100, null=True, blank=True, verbose_name="Menu Name")
    menuImageText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Icon Text")
    brand = models.ForeignKey(
        Brands, on_delete=models.SET_NULL, null=True,blank=True, related_name="br_page", verbose_name="Brand")
    
    # Not Used
    tags = ClusterTaggableManager(through=CategoryPageTag, blank=True)
    menuImage = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.SET_NULL, related_name='+', blank=True, null=True, verbose_name="Menu Image"
    )
    brShortDesc = RichTextField(
        blank=True, null=True, verbose_name="Short Description")
    
    subpage_types = [
        'ecom.ProductPage',  # appname.ModelName
    ]

    content_panels = Page.content_panels + [
        MultiFieldPanel([
            FieldPanel('brand'),
            # SubcategoryChooser(
            #         "brand",
            #         category=Company,
            #         subcategory=Brands,
            #         subcategory_related_name="brands",
            #         search_text="Search Brands"
            # ),
            FieldRowPanel([
                FieldPanel('search_image',heading="Thumb Image"),
                FieldPanel('show_footer',widget=forms.CheckboxInput())
            ]),
            FieldRowPanel([
                FieldPanel('menuName'),
                FieldPanel('menuImageText',heading="Menu Icon Text"),
            ]),
        ]),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('bannerHeight'),
                FieldPanel('bannerWidth'),
                FieldPanel('bannerTimer'),
            ]),
            FieldPanel('bannercontent')
        ], heading="Banner"),
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(Page.promote_panels, heading='Promote'),
    ])

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def mobile_breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs[4:]

    def get_absolute_url(self):
        return f'{self.url}'
    def get_total_product(self):
        totalProdList=[]
        for product in self.get_children().live():
            for var in product.get_children().live():
                if var.specific.thumb:
                    totalProdList.append(var)
        return totalProdList
        
    def get_context(self, request):
        context = super().get_context(request)
        prodPages=self.get_children().live()
        totalProdList=[]
        for product in prodPages:
            for var in product.get_children().live():
                if var.specific.thumb:
                    totalProdList.append(var)
        context['total_products'] = totalProdList
        
        return context

    
    # @re_path(r'^<cat>/$')
    # @path('')
    # @path('<subcat>/')
    # def all_prod(self, request,subcat=None):
    #     if subcat:
    #         print("Route Query=",subcat)
    #         # try:
    #         events = ProductPage.objects.filter(cat__slug=subcat)
    #         # except:
    #             # events = CategoryPage.objects.get(title=subcat).get_children()

    #     else:
    #         events = ProductPage.objects.filter(cat__slug=self.title)

    #     print("Result Set=",events)

    #     return self.render(request, context_overrides={
    #         'events': events,
    #         'title':subcat
    #     },)

    # @path('')
    # def all_prod(self, request):
    #     events = ProductPage.objects.filter(cat__cat__name=self.title)
    #     # return self.render(request,'pages/search_result.html',{'prodObj':events})
    #     return self.render(request, context_overrides={
    #         'events': events,
    #     },)

    class Meta:
        verbose_name = "Product Brand"


class Offers(models.Model):
    fromDate=models.DateTimeField(null=True)
    toDate=models.DateTimeField(null=True)
    name=models.CharField(null=True,max_length=200)
    def __str__(self):
        return self.name
    
# Product

class ProductPageTag(ItemBase):
    tag = models.ForeignKey(
        CustomTag, related_name="tagged_product", on_delete=models.CASCADE
    )
    content_object = ParentalKey(
        'ProductPage',
        related_name='tagged_items',
        on_delete=models.CASCADE
    )

from itertools import groupby

class ProductPage(RoutablePageMixin, MetadataPageMixin, AbstractCarousel, Page, ClusterableModel, AbstractCustomColor, HitCountMixin):
    template="DetailPage/ecom/ProductDetail.html"
    subpage_types = [
        'ecom.ProductVariantPage',
    ]
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    customTitle = RichTextField(verbose_name="Custom Title", null=True)
    color=ColorField(default="#FFFFFF",verbose_name="Color",blank=True,null=True)
    colorImg = models.ForeignKey(
        "wagtailimages.Image",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name="Color Image"
    )
    colorName=models.CharField(max_length=150,blank=True,null=True,verbose_name="Color Name")
    subCat = models.ForeignKey(
        SubCategory, on_delete=models.SET_NULL, null=True,blank=True, related_name="cat_products", verbose_name="Category")
    
    prBrand = models.ForeignKey(
        Brands, on_delete=models.SET_NULL, null=True,blank=True, related_name="br_products", verbose_name="Brand")
    
    prodSeries = models.CharField(max_length=150, blank=True, null=True)
    
    prodMRP = models.DecimalField(
        verbose_name="MRP", decimal_places=2, max_digits=10, blank=True, null=True)
    prodDealPrice = models.DecimalField(
        verbose_name="Deal Price", decimal_places=2, max_digits=10, blank=True, null=True)
    prodRating = models.DecimalField(default=0, decimal_places=1, max_digits=5)
    
    tags = ClusterTaggableManager(through=ProductPageTag, blank=True)
    prDesc = CKEditor5Field(
        'Description', config_name='extends', blank=True, null=True)
    prRec = models.ForeignKey(
        Recommendation, on_delete=models.SET_NULL, null=True, blank=True, related_name="rec_products", verbose_name="Recommendation")
    launchDate = models.DateTimeField(
        blank=True, null=True, verbose_name="Launch Date")
    prodStory = StreamField([
        ('web_story_page', WebStoryBlock(label="Story Page",
         block_counts={'image': {'max_num': 1}})),
    ], use_json_field=True, blank=True, null=True)

    # Not Used
    menuName = models.CharField(max_length=100, blank=True, null=True)
    # body = StreamField([
    #     ('section', SectionBlock()),
    #     ('row', RowBlock()),
    # ], null=True, blank=True, use_json_field=True)
    show_footer = models.BooleanField(blank=True, null=True, default=True)
    # Not Used /
    
    # modify your content_panels:
    content_panels = Page.content_panels + [
        FieldPanel('customTitle'),
        FieldPanel('search_image', heading="Product Title Image"),
       
        MultiFieldPanel([
            FieldRowPanel([
                SubcategoryChooser(
                    "subCat",
                    category=Category,
                    subcategory=SubCategory,
                    subcategory_related_name="sub_categories",
                    search_text="Search Categories"
                ),
                FieldPanel('prBrand')
                # SubcategoryChooser(
                #     "prBrand",
                #     category=Company,
                #     subcategory=Brands,
                #     subcategory_related_name="brands",
                #     search_text="Search Brands"
                # )
            ]),
            FieldRowPanel([
                FieldPanel('prodSeries', heading="Product Series"),
                FieldPanel('prodRating', heading="Rating")
            ]),
            # FieldRowPanel([
            #     FieldPanel('prodMRP', widget=forms.NumberInput(
            #         attrs={'id': 'prodMRP'})),
            #     FieldPanel('prodDealPrice'),
            # ]),
            
            FieldRowPanel([
                FieldPanel('prRec'),
                FieldPanel('launchDate'),
            ]),
           
            FieldPanel('tags'),
        ]),
        # FieldPanel('prDesc'),
        MultiFieldPanel([
            InlinePanel('prod_offer', label="Product Offer"),
            
        ])
    ]
    # social_panel = [
    #     InlinePanel('prod_social'),
    # ]
    attr_panel = [
        InlinePanel('prod_attr_values', label="Attributes"),
    ]
    related_panel = [
        InlinePanel('related_products')
    ]
    acce_panel = [
        InlinePanel('product_accesories')
    ]
    banner_panel = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('bannerHeight'),
                FieldPanel('bannerWidth'),
                FieldPanel('bannerTimer'),
            ]),
            FieldPanel('bannercontent')
        ])
    ]
    story_panels = [
        FieldPanel('prodStory', heading="Web Story")
    ]
    apps_panel = [
        InlinePanel('prod_app_item', label="App")
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(banner_panel, heading='Banner'),
        # ObjectList(deals_panel, heading='Deals'),
        ObjectList(attr_panel, heading='Attributes'),
        # ObjectList(social_panel, heading='Social'),
        ObjectList(acce_panel, heading='Accesories'),
        ObjectList(related_panel, heading='Similar'),
        ObjectList(apps_panel, heading='Apps'),
        ObjectList(story_panels, heading='Story'),
        ObjectList(Page.promote_panels, heading='Promote'),
    ])

    search_fields = Page.search_fields + [
        index.SearchField('title'),
        index.SearchField('slug'),
        index.SearchField('customTitle'),
        index.SearchField('subCat'),
        index.SearchField('prBrand'),
        index.SearchField('tags'),
        index.SearchField('prDesc'),
        index.SearchField('prodDealPrice'),
        index.SearchField('prodMRP'),

        index.FilterField('title'),
        index.FilterField('slug'),
        index.FilterField('customTitle'),
        index.FilterField('subCat'),
        index.FilterField('prBrand'),
        index.FilterField('tags'),
        index.FilterField('prDesc'),
        index.FilterField('prodDealPrice'),
        index.FilterField('prodMRP'),
    ]

    # base_form_class = ProductPageForm
    @property
    def is_offer_due(self):
        today=datetime.now()
        # print("Time=",today.time() < self.prOffer.fromDate.time())
        if today.time() <= self.prOffer.fromDate.time() and today.time() < self.prOffer.toDate.time():
            return True
        else:
            return False
        
    @property
    def is_past_due(self):
        if self.launchDate:
            return datetime.now().astimezone(timezone('UTC')) < self.launchDate
        else:
            return False

    @property
    def parent_event(self):
        parent_page = self.get_parent()
        self.specific.title = parent_page.specific.title
        # if parent_page.specific_class == Event:
        #     return parent_page.specific  # or just `return parent_page` if you don't need the extra detail from that page
        # else:
        #     return None

    @path('add-review/')
    def add_review(self, request):
        if request.POST['rating']:
            stars = request.POST['rating']
        else:
            stars = 1
        ProductReview.objects.create(
            stars=stars, content=request.POST['content'], user=request.user, product=self)
        return HttpResponseRedirect(self.url)

    @path('notify-me/')
    def notify_me(self, request):
        ProdNotification.objects.create(user=request.user, product=self)
        return HttpResponseRedirect(self.url)

    @re_path(r'^redmi/$')
    def events_for_year(self, request):
        events = ProductPage.objects.live().filter(title__icontains="Redmi")
        return self.render(request, 'pages/search_result.html', {'prodObj': events})

        # return self.render(request, context_overrides={
        #     'events': events,
        # })

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def mobile_breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs[5:]

    def get_absolute_url(self):
        return f'{self.url}'

    def get_rating(self):
        total = sum(int(review['stars']) for review in self.reviews.values())
        if self.reviews.count() > 0:
            return int(total / self.reviews.count())
        else:
            return 0

    def get_features(self):
        # try:
        #     if self.get_parent().prod_attr_values:
        #         featuresObj = []
        #         for baseAttr in self.get_parent().prod_attr_values.all():
        #             for varAttr in self.prod_attr_values.all():
        #                 if baseAttr.attribute.name == varAttr.attribute.name:
        #                     # print("Base Attr=",baseAttr.attribute.id,varAttr.attribute.id)
        #                     if varAttr.value_text:
        #                         pass
        #                     else:
        #                         featuresObj.append(baseAttr)
        #                 if varAttr.value_text:
        #                     featuresObj.append(varAttr)
        #     else:
        #         featuresObj = self.prod_attr_values.all()
        # except:
        featuresObj = self.prod_attr_values.all()
        return list(set(featuresObj))

    def get_thumb_image(self):
        return self.search_image
    def get_buy_now(self):
        # if self.prod_social.count() > 0:
        #     return self.prod_social.first().url
        # else:
        return None
    
    def get_default(self):
        for var in self.get_children().live():
            if var.specific.thumb:
                return var
    
    def get_context(self, request, *args, **kwargs):
        context = super().get_context(request, *args, **kwargs)
        context['product_attributes'] = json.dumps(
            {attr.attribute.id: str(attr.attribute)
             for attr in self.prod_attr_values.all()},
            cls=DjangoJSONEncoder
        )
        context['blogs'] = self.subCat.cat.blogs.all()[:10]
        context['news'] = self.subCat.cat.cat_news.all()[:10]
        prodAttrs=[]
        similarProducts=[]
        if self.related_products.all():
            for rel in self.related_products.all():
                similarProducts.append(rel.related)
            similarProducts.insert(0,self)
            for pa in similarProducts:
                for attr in ProductAttributeValue.objects.filter(product=pa):
                    if attr.attribute.attr:
                        prodAttrs.append(attr.attribute.attr.name)
        context['similarProducts']=similarProducts
        
        
        features=self.prod_attr_values.all()
        
        context['variants']=self.get_children().live()
        return context
    class Meta:
        verbose_name = "Product Detail Page"

class ProductAppItems(Orderable):
    product = ParentalKey(ProductPage, related_name="prod_app_item")
    item = models.ForeignKey(
        AppItems,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    panels=[
        SubcategoryChooser(
            "item",
            category=App,
            subcategory=AppItems,
            subcategory_related_name="app_items",
            search_text="Search items"
        ),
        
    ]

class ProductAttributeValue(Orderable):
    product = ParentalKey(ProductPage, related_name="prod_attr_values")
    attribute = models.ForeignKey(ProductAttributes, on_delete=models.CASCADE)
    value_text = models.CharField(
        max_length=250, verbose_name="Value", blank=True, null=True)
    value_text_opt = models.CharField(
        max_length=250, verbose_name="Value Optional", blank=True, null=True)
    common=models.BooleanField(default=False,null=True,blank=True)

    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel("attribute", read_only=True),
                FieldPanel("value_text"),
            ]),
            FieldPanel("common",heading="Highlight",widget=forms.CheckboxInput())
        ])
    ]

    def __str__(self):
        return str(self.attribute)

    class Meta:
        verbose_name_plural = "Product Attribute Value"

class ProductReview(models.Model):
    product = models.ForeignKey(
        ProductPage, related_name='reviews', on_delete=models.CASCADE)
    user = models.ForeignKey(
        User, related_name='reviews', on_delete=models.CASCADE)
    content = models.TextField(blank=True, null=True)
    stars = models.IntegerField()
    date_added = models.DateTimeField(auto_now_add=True)

class ProductAttributeFieldPanel(FieldPanel):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.product_page = None

    # def on_model_bound(self):
    #     super().on_model_bound()
    #     self.product_page = self.model.prodVar.product

    def get_form_options(self):
        options = super().get_form_options()
        if self.product_page:
            options['widgets'] = {
                'attribute': forms.Select(choices=self.get_product_attributes())
            }
        return options

    def get_product_attributes(self):
        # print("Productage=", self.product_page)
        if self.product_page:
            return [(attr.attribute.id, str(attr.attribute)) for attr in self.product_page.prod_attr_values.all()]
        return []

class ProductOffer(Orderable):
    product = ParentalKey(ProductPage, related_name="prod_offer")
    offer = models.ForeignKey(
        Offers,
        null=True,
        on_delete=models.SET_NULL,
        verbose_name="Offer",
        blank=True,related_name="off_products"
    )
    offerPrice = models.DecimalField(
        verbose_name="Offer Price", decimal_places=2, max_digits=10, blank=True, null=True)
    offerTitle = models.CharField(
        max_length=200, verbose_name="Title", null=True, blank=True)
    offerDiscount = models.DecimalField(decimal_places=2, max_digits=4, blank=True, null=True)
    scPlateform = models.ForeignKey(
        SocialPlatform, on_delete=models.SET_NULL, null=True, related_name="prod_deal_platform")
    scURL = models.URLField(verbose_name="URL", null=True)
    offerDetail = CKEditor5Field(
        'Offer Detail', config_name='extends', blank=True, null=True)
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('offer'),
                FieldPanel('offerPrice'),
            ]),
            FieldPanel('offerDetail'),
            # FieldRowPanel([
            #     FieldPanel('offerDiscount', heading="Discount(%)",
            #                widget=forms.TextInput(attrs={'id': 'dealDisc', 'onkeyup': 'dealPerc()'})),
            #     FieldPanel('scPrice', widget=forms.NumberInput(
            #         attrs={'id': 'dealPrice', 'onkeyup': 'price()'})),
            # ]),
        ])
    ]

    def __str__(self):
        return str(self.offer)

    class Meta:
        verbose_name_plural = "Product Offer"


class RelatedProduct(Orderable):
    product = ParentalKey(ProductPage, related_name="related_products")
    related=models.ForeignKey(ProductPage,on_delete=models.CASCADE)
    def __str__(self):
        return str(self.related)
    class Meta:
        verbose_name_plural = "Related Product"
        
class ProductAccesories(Orderable):
    product = ParentalKey(ProductPage, related_name="product_accesories")
    related=models.ForeignKey(ProductPage,on_delete=models.CASCADE)
    def __str__(self):
        return str(self.related)
    class Meta:
        verbose_name_plural = "Product Accesories"

@register_snippet
class ProdNotification(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    product = models.ForeignKey(ProductPage, on_delete=models.CASCADE)
    date = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return str(self.product)



class ProductVariantPage(RoutablePageMixin, MetadataPageMixin, AbstractCarousel, Page, ClusterableModel, AbstractCustomColor, HitCountMixin):
    template="DetailPage/ecom/ProductVarDetail.html"
    subpage_types = [
    ]
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    customTitle = RichTextField(verbose_name="Custom Title", null=True,blank=True)
    color=ColorField(default="#000000",verbose_name="Color",blank=True,null=True)
    colorImg = models.ForeignKey(
        "wagtailimages.Image",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name="Color Image"
    )
    colorName=models.CharField(max_length=150,blank=True,null=True,verbose_name="Color Name")
    
    prodSeries = models.CharField(max_length=150, blank=True, null=True)
    prodMRP = models.DecimalField(
        verbose_name="MRP", decimal_places=2, max_digits=10, blank=True, null=True)
    prodDealPrice = models.DecimalField(
        verbose_name="Deal Price", decimal_places=2, max_digits=10, blank=True, null=True)
    prodRating = models.DecimalField(default=0, decimal_places=1, max_digits=5)
    prDesc = CKEditor5Field(
        'Description', config_name='extends', blank=True, null=True)
    prRec = models.ForeignKey(
        Recommendation, on_delete=models.SET_NULL, null=True, blank=True, related_name="rec_var_products", verbose_name="Recommendation")
    launchDate = models.DateTimeField(
        blank=True, null=True, verbose_name="Launch Date")
    prodStory = StreamField([
        ('web_story_page', WebStoryBlock(label="Story Page",
         block_counts={'image': {'max_num': 1}})),
    ], use_json_field=True, blank=True, null=True)
    thumb=models.BooleanField(blank=True,null=True,verbose_name="View Thumbnail ( Default )")
    # Not Used
    
    # modify your content_panels:
    content_panels = Page.content_panels + [
        FieldPanel('customTitle'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('search_image', heading="Product Title Image"),
                FieldPanel('thumb',widget=forms.CheckboxInput()),
            ])
        ]),
        MultiFieldPanel([
            FieldRowPanel([
                NativeColorPanel('color'),
                FieldPanel('colorImg'),
                FieldPanel('colorName')
            ])
        ],heading="Color"),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('prodSeries', heading="Product Series"),
                FieldPanel('prodRating', heading="Rating")
            ]),
            FieldRowPanel([
                FieldPanel('prodMRP', widget=forms.NumberInput(
                    attrs={'id': 'prodMRP'})),
                FieldPanel('prodDealPrice'),
            ]),
           
            InlinePanel('prod_var_images', label="Images"),
        ]),
        FieldPanel('prDesc'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('prRec'),
                FieldPanel('launchDate'),
            ])
        ])
    ]
    social_panel = [
        InlinePanel('prod_var_social'),
    ]
    attr_panel = [
        InlinePanel('prod_var_attr_values', label="Attributes"),
    ]
    banner_panel = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('bannerHeight'),
                FieldPanel('bannerWidth'),
                FieldPanel('bannerTimer'),
            ]),
            FieldPanel('bannercontent')
        ])
    ]
   
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(attr_panel, heading='Attributes'),
        ObjectList(social_panel, heading='Social'),
        ObjectList(Page.promote_panels, heading='Promote'),
        ObjectList(banner_panel, heading='Banner')
    ])

    search_fields = Page.search_fields + [
        index.SearchField('title'),
        index.SearchField('slug'),
        index.SearchField('customTitle'),
        index.SearchField('prDesc'),
        index.SearchField('prodDealPrice'),
        index.SearchField('prodMRP'),

        index.FilterField('title'),
        index.FilterField('slug'),
        index.FilterField('customTitle'),
        index.FilterField('prDesc'),
        index.FilterField('prodDealPrice'),
        index.FilterField('prodMRP'),
    ]

    # base_form_class = ProductPageForm
    @property
    def is_offer_due(self):
        today=datetime.now()
        # print("Time=",today.time() < self.prOffer.fromDate.time())
        if today.time() <= self.prOffer.fromDate.time() and today.time() < self.prOffer.toDate.time():
            return True
        else:
            return False
        
    @property
    def is_past_due(self):
        if self.launchDate:
            return datetime.now().astimezone(timezone('UTC')) < self.launchDate
        else:
            return False

    # @property
    # def parent_event(self):
    #     parent_page = self.get_parent()
    #     self.specific.title = parent_page.specific.title

    @path('add-review/')
    def add_review(self, request):
        if request.POST['rating']:
            stars = request.POST['rating']
        else:
            stars = 1
        ProductReview.objects.create(
            stars=stars, content=request.POST['content'], user=request.user, product=self)
        return HttpResponseRedirect(self.url)

    @path('notify-me/')
    def notify_me(self, request):
        ProdNotification.objects.create(user=request.user, product=self)
        return HttpResponseRedirect(self.url)

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def mobile_breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs[5:]

    def get_absolute_url(self):
        return f'{self.url}'

    def get_rating(self):
        total = sum(int(review['stars']) for review in self.reviews.values())
        if self.reviews.count() > 0:
            return int(total / self.reviews.count())
        else:
            return 0

   
    def get_buy_now(self):
        if self.prod_var_social.count() > 0:
            return self.prod_var_social.first().url
        else:
            return None
    
    def get_price(self):
        context={}
        if self.prodMRP:
            context['mrp']=self.prodMRP
        else:
            context['mrp']=self.get_parent().specific.prodMRP
            
        if self.prodDealPrice:
            context['dealPrice']=self.prodDealPrice
        else:
            context['dealPrice']=self.get_parent().specific.prodDealPrice
        return context
        
    def get_title(self):
        return str(self.get_parent().title)+" "+str(self.title)
    def get_context(self, request, *args, **kwargs):
        context = super().get_context(request, *args, **kwargs)
        context['blogs'] = self.get_parent().specific.subCat.cat.blogs.all()[:10]
        context['news'] = self.get_parent().specific.subCat.cat.cat_news.all()[:10]
        variants=[]
        variants.append(self.get_parent())
        for var in self.get_parent().get_children().live():
            variants.append(var)
            
        if self.get_siblings().live().count() > 1:
            context['variants']=self.get_siblings().live()
        
        if self.prodMRP:
            context['mrp']=self.prodMRP
        else:
            context['mrp']=self.get_parent().specific.prodMRP
            
        if self.prodDealPrice:
            context['dealPrice']=self.prodDealPrice
        else:
            context['dealPrice']=self.get_parent().specific.prodDealPrice
        
        featuresList=[]
        for pav in self.prod_var_attr_values.all():
            if pav.value_text == None:
                if self.get_parent().specific.prod_attr_values.filter(attribute=pav.attribute).exists():
                    if self.get_parent().specific.prod_attr_values.get(attribute=pav.attribute).value_text:
                        featuresList.append(self.get_parent().specific.prod_attr_values.get(attribute=pav.attribute))
            else:
                featuresList.append(pav)
        featuresObj = self.prod_var_attr_values.all()
        context['features']=featuresList
        # context['features']=list(set(featuresObj))
        
        prodAttrs=[]
        similarProducts=[]
        if self.get_parent().specific.related_products.all():
            for rel in self.get_parent().specific.related_products.all():
                if rel.related.content_type.model == "productpage":
                    for pv in rel.related.get_children().live():
                        if pv.specific.thumb:
                            similarProducts.append(ProductVariantPage.objects.get(title=pv.title))
                            # print("similar=",pv,pv.specific.thumb)
            similarProducts.insert(0,self)
            for pa in similarProducts:
                for attr in ProductVariantAttributeValue.objects.filter(product=pa):
                    if attr.attribute.attr:
                        prodAttrs.append(attr.attribute.attr.name)
        context['similarProducts']=list(set(similarProducts))
        context['similarAttr']=list(set(prodAttrs))
        
        
        commonfeaturesList=[]
        for pav in self.get_parent().specific.prod_attr_values.all():
            if pav.value_text != None and pav.common == True:
                commonfeaturesList.append(pav)
        context['commonfeaturesList']=commonfeaturesList
        
        
        return context
    
    
    
    
    
    class Meta:
        verbose_name = "Product Variant Page"

class ProductVarImage(Orderable):
    product = ParentalKey(ProductVariantPage, related_name="prod_var_images")
    image = models.ForeignKey(
        "wagtailimages.Image",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name=""
    )
    class Meta:
        verbose_name_plural = "Product Image"

class ProductVarSocial(Orderable):
    product = ParentalKey(
        ProductVariantPage, related_name='prod_var_social')
    platform = models.ForeignKey(
        SocialPlatform, on_delete=models.CASCADE, null=True)
    url = models.URLField(blank=True, null=True)

    def __str__(self):
        return str(self.product)

class ProductVariantAttributeValue(Orderable):
    product = ParentalKey(ProductVariantPage, related_name="prod_var_attr_values")
    attribute = models.ForeignKey(ProductAttributes, on_delete=models.CASCADE)
    value_text = models.CharField(
        max_length=250, verbose_name="Value", blank=True, null=True)
    value_text_opt = models.CharField(
        max_length=250, verbose_name="Value Optional", blank=True, null=True)
    common=models.BooleanField(default=False,null=True,blank=True)
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel("attribute", read_only=True),
                FieldPanel("value_text"),
                # FieldPanel("aggt"),
            ])
            # FieldPanel("common",heading="Common",widget=forms.CheckboxInput())
        ])
    ]
    def __str__(self):
        return str(self.attribute)
    class Meta:
        verbose_name_plural = "Product Variant Attribute Value"

# from .signals import *